@@ -196,7 +196,7 @@ io_error (SoupSocket *sock, SoupMessage *msg, GError *error)
 		   !g_error_matches (error, G_IO_ERROR, G_IO_ERROR_TIMED_OUT) &&
 		   request_is_idempotent (msg)) {
 		/* Connection got closed, but we can safely try again */
-		io->item->state = SOUP_MESSAGE_STARTING;
+		io->item->state = SOUP_MESSAGE_RESTARTING;
 	} else if (!SOUP_STATUS_IS_TRANSPORT_ERROR (msg->status_code))
 		soup_message_set_status (msg, SOUP_STATUS_IO_ERROR);
 
@@ -309,8 +309,26 @@ read_metadata (SoupMessage *msg, gboolean to_blank)
 			g_byte_array_append (io->read_meta_buf, read_buf, nread);
 			break;
 
-		case SOUP_SOCKET_ERROR:
 		case SOUP_SOCKET_EOF:
+			/* More lame server handling... deal with
+			 * servers that don't send the final chunk.
+			 */
+			if (io->read_state == SOUP_MESSAGE_IO_STATE_CHUNK_SIZE &&
+			    io->read_meta_buf->len == 0) {
+				g_byte_array_append (io->read_meta_buf,
+						     (guchar *)"0\r\n", 3);
+				got_lf = TRUE;
+				break;
+			} else if (io->read_state == SOUP_MESSAGE_IO_STATE_TRAILERS &&
+				   io->read_meta_buf->len == 0) {
+				g_byte_array_append (io->read_meta_buf,
+						     (guchar *)"\r\n", 2);
+				got_lf = TRUE;
+				break;
+			}
+			/* else fall through */
+
+		case SOUP_SOCKET_ERROR:
 			io_error (io->sock, msg, error);
 			return FALSE;
 
